/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <app.h>
#include <Elementary.h>
#include <app_debug.h>
#include <viewmgr.h>
#include <key_define.h>
#include <Ecore_Wayland.h>

#include "defs.h"
#include "view.h"
#include "utils.h"

#define VALUE_INFOSQUARE "infosquare"

SET_TAG(PACKAGE);

struct _appdata {
	const char *name;

	Evas_Object *win;
	Eina_Bool activated;
	Ecore_Event_Handler *key_down;
	Ecore_Event_Handler *focus_out;
};

static Evas_Object *_add_win(const char *name)
{
	Evas_Object *win;

	win = elm_win_add(NULL, name, ELM_WIN_BASIC);
	if (!win)
		return NULL;

	elm_win_title_set(win, name);
	elm_win_alpha_set(win, EINA_TRUE);
	evas_object_show(win);

	return win;
}

static void _hide(struct _appdata *ad)
{
	viewmgr_hide_view(VIEW_HOME);
	viewmgr_hide_view(VIEW_RECENT);

	elm_win_iconified_set(ad->win, EINA_TRUE);
	ad->activated = EINA_FALSE;
}

static void _show(struct _appdata *ad)
{
	elm_win_activate(ad->win);
	ad->activated = EINA_TRUE;

	viewmgr_push_view(VIEW_HOME);
}

static Eina_Bool _focus_out(void *data, int type, void *event)
{
	if (!data) {
		_ERR("Invalid argument");
		return ECORE_CALLBACK_CANCEL;
	}

	_hide(data);

	return ECORE_CALLBACK_RENEW;
}

static Eina_Bool _key_down(void *data, int type, void *ei)
{
	struct _appdata *ad;
	Evas_Event_Key_Down *ev;

	if (!data || !ei) {
		_ERR("Invalid argument");
		return ECORE_CALLBACK_CANCEL;
	}

	ev = ei;
	ad = data;

	if (!strcmp(ev->keyname, KEY_POWER_REMOTE)) {
		utils_launch_app(PACKAGE_INFOSQUARE, KEY_INFOSQUARE,
				VALUE_INFOSQUARE);
	} else if (!strcmp(ev->keyname, KEY_TV_REMOTE)) {
		utils_launch_app(PACKAGE_LIVETV, NULL, NULL);
	} else if (!strcmp(ev->keyname, KEY_HOME)) {
		if (!ad->activated)
			_show(ad);
	} else if (!strcmp(ev->keyname, KEY_BACK) ||
			!strcmp(ev->keyname, KEY_BACK_REMOTE)) {
		if (ad->activated && viewmgr_active_view_count() == 1 &&
				viewmgr_get_view_state(VIEW_HOME) ==
						VIEW_STATE_VISIBLE)
			_hide(ad);
	}

	return ECORE_CALLBACK_CANCEL;
}

/* FIXME: Keygrab Hot Fix */
static Eina_Bool _key_timer(void *data)
{
	if (!data) {
		_ERR("Invalid argument");
		return ECORE_CALLBACK_CANCEL;
	}

	elm_win_keygrab_set(data, KEY_TV_REMOTE, 0, 0, 0,
			ELM_WIN_KEYGRAB_EXCLUSIVE);
	elm_win_keygrab_set(data, KEY_POWER_REMOTE, 0, 0, 0,
			ELM_WIN_KEYGRAB_EXCLUSIVE);
	elm_win_keygrab_set(data, KEY_HOME, 0, 0, 0, ELM_WIN_KEYGRAB_EXCLUSIVE);

	return ECORE_CALLBACK_CANCEL;
}

static bool _create(void *user_data)
{
	struct _appdata *ad;

	if (!user_data) {
		_ERR("Invalid argument");
		return false;
	}

	ad = user_data;

	elm_theme_overlay_add(NULL, THEMEFILE);
	elm_config_focus_move_policy_set(ELM_FOCUS_MOVE_POLICY_CLICK);

	ad->win = _add_win(ad->name);
	if (!ad->win) {
		_ERR("failed to create window");
		return false;
	}
	ecore_timer_add(3, _key_timer, ad->win);

	if (!viewmgr_create(ad->win)) {
		_ERR("failed to create viewmgr");
		evas_object_del(ad->win);
		ad->win = NULL;
		return false;
	}

	viewmgr_add_view(view_home_get_vclass(), NULL);
	viewmgr_add_view(view_recent_get_vclass(), NULL);
	viewmgr_add_view(view_user_get_vclass(), NULL);

	ad->key_down = ecore_event_handler_add(ECORE_EVENT_KEY_DOWN, _key_down,
			ad);
	ad->focus_out = ecore_event_handler_add(ECORE_WL_EVENT_FOCUS_OUT,
			_focus_out, ad);

	return true;
}

static void _terminate(void *user_data)
{
	struct _appdata *ad;

	if (!user_data)
		return;

	ad = user_data;

	if (ad->win) {
		if (ad->key_down)
			ecore_event_handler_del(ad->key_down);
		if (ad->focus_out)
			ecore_event_handler_del(ad->focus_out);

		elm_win_keygrab_unset(ad->win, KEY_POWER_REMOTE, 0, 0);
		elm_win_keygrab_unset(ad->win, KEY_TV_REMOTE, 0, 0);
		elm_win_keygrab_unset(ad->win, KEY_HOME, 0, 0);

		viewmgr_destroy();
		evas_object_del(ad->win);
	}
}

static void _pause(void *user_data)
{

}

static void _resume(void *user_data)
{

}

static void _control(app_control_h app_control, void *user_data)
{
	struct _appdata *ad;

	if (!user_data) {
		_ERR("Invalid argument");
		return;
	}

	ad = user_data;

	if (ad->win)
		_show(ad);
}

int main(int argc, char **argv)
{
	struct _appdata ad;

	ui_app_lifecycle_callback_s event_callback = {0,};

	event_callback.create = _create;
	event_callback.terminate = _terminate;
	event_callback.pause = _pause;
	event_callback.resume = _resume;
	event_callback.app_control = _control;

	memset(&ad, 0x00, sizeof(ad));
	ad.name = PACKAGE;

	return ui_app_main(argc, argv, &event_callback, &ad);
}

