/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <app_debug.h>
#include <viewmgr.h>
#include <inputmgr.h>

#include "defs.h"
#include "view.h"
#include "utils.h"

#define ACTION_MENU_TITLE_RESET "Reset"

enum input_handler_type {
	INPUT_HANDLER_BTN,
	INPUT_HANDLER_ITEM
};

struct _priv {
	Evas_Object *win;
	Evas_Object *base;
	Evas_Object *btn;
	Evas_Object *item;
};

static void _focused(int id, void *data, Evas_Object *obj,
		Elm_Object_Item *item)
{
	elm_object_signal_emit(obj, SIG_FOCUS, SRC_PROG);
}

static void _unfocused(int id, void *data, Evas_Object *obj,
		Elm_Object_Item *item)
{
	elm_object_signal_emit(obj, SIG_UNFOCUS, SRC_PROG);
}

static void _select(int id, void *data, Evas_Object *obj)
{
	/* It should be implemented later */

	switch (id) {
	case INPUT_HANDLER_BTN:
		break;
	case INPUT_HANDLER_ITEM:
		break;
	default:
		return;
	}
}

static void _key_down(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Down *ev)
{
	if (!strcmp(ev->keyname, KEY_BACK) ||
			!strcmp(ev->keyname, KEY_BACK_REMOTE)) {
		viewmgr_hide_view(VIEW_ACTION_MENU);
	} else if (!strcmp(ev->keyname, KEY_ENTER) ||
			!strcmp(ev->keyname, KEY_ENTER_REMOTE)) {
		_select(id, data, obj);
	}
}

static void _mouse_down(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Down *ev)
{
	_select(id, data, obj);
}

static void _mouse_move(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Move *ev)
{
	if (!elm_object_focus_get(obj))
		elm_object_focus_set(obj, EINA_TRUE);
}

static input_handler handler = {
	.focused = _focused,
	.unfocused = _unfocused,
	.key_down = _key_down,
	.mouse_down = _mouse_down,
	.mouse_move = _mouse_move
};

static Evas_Object *_add_item(Evas_Object *box, struct _priv *priv)
{
	Evas_Object *item, *ic, *focus_ic, *lbl, *focus_lbl, *bg;

	item = utils_add_layout(box, GRP_ACTION_MENU_TABLE_ITEM, true, NULL);
	if (!item) {
		_ERR("failed to add item");
		evas_object_del(box);
		return false;
	}

	ic = utils_add_icon(item, IMAGE_ACTION_MENU_ICON_RESET,
			PART_ACTION_MENU_TABLE_ITEM_ICON);
	if (!ic)
		goto err;

	focus_ic = utils_add_icon(item, IMAGE_ACTION_MENU_ICON_RESET_FOCUS,
			PART_ACTION_MENU_TABLE_ITEM_ICON_FOCUS);
	if (!focus_ic)
		goto err;

	lbl = utils_add_label(item, ACTION_MENU_TITLE_RESET,
			STYLE_LABEL_ACTION_MENU_TITLE,
			PART_ACTION_MENU_TABLE_ITEM_TEXT);
	if (!lbl)
		goto err;

	focus_lbl = utils_add_label(item, ACTION_MENU_TITLE_RESET,
			STYLE_LABEL_ACTION_MENU_TITLE_FOCUS,
			PART_ACTION_MENU_TABLE_ITEM_TEXT_FOCUS);
	if (!focus_lbl)
		goto err;

	bg = utils_add_bg(item, COLOR_DEFAULT_R, COLOR_DEFAULT_G,
			COLOR_DEFAULT_B, COLOR_DEFAULT_A,
			PART_ACTION_MENU_TABLE_ITEM_BG);
	if (!bg)
		goto err;

	evas_object_show(item);
	inputmgr_add_callback(item, INPUT_HANDLER_ITEM, &handler, priv);

	return item;
err:
	evas_object_del(item);
	return NULL;
}

static bool _add_menu(struct _priv *priv)
{
	Evas_Object *btn, *box, *item;

	btn = utils_add_layout(priv->base, GRP_ACTION_MENU_LIVETV, true,
			PART_ACTION_MENU_LIVETV);
	if (!btn) {
		_ERR("failed to add btn");
		return false;
	}
	inputmgr_add_callback(btn, INPUT_HANDLER_BTN, &handler, priv);
	priv->btn = btn;

	box = utils_add_box(priv->base, true);
	if (!box) {
		_ERR("failed to add box");
		return false;
	}
	elm_object_part_content_set(priv->base, PART_ACTION_MENU_TABLE, box);
	elm_box_align_set(box, 0.0, 0.5);

	item = _add_item(box, priv);
	if (!item) {
		_ERR("failed to add item");
		evas_object_del(box);
		return false;
	}

	elm_box_pack_start(box, item);
	priv->item = item;

	return true;
}

static Evas_Object *_create(Evas_Object *win, void *data)
{
	struct _priv *priv;
	Evas_Object *base;

	priv = calloc(1, sizeof(*priv));
	if (!priv) {
		_ERR("failed to calloc priv");
		return NULL;
	}

	base = utils_add_layout(win, GRP_ACTION_MENU, false, NULL);
	if (!base) {
		_ERR("failed to create base");
		free(priv);
		return NULL;
	}
	evas_object_size_hint_weight_set(base, EVAS_HINT_EXPAND,
			EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, base);

	priv->win = win;
	priv->base = base;

	if (!_add_menu(priv)) {
		_ERR("failed to add menu");
		free(priv);
		evas_object_del(base);
		return NULL;
	}

	viewmgr_set_view_data(VIEW_ACTION_MENU, priv);

	return base;
}

static void _show(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	evas_object_show(priv->base);

	if (priv->btn)
		elm_object_focus_set(priv->btn, EINA_TRUE);
}

static void _hide(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	evas_object_hide(priv->base);
}

static void _destroy(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	evas_object_del(priv->base);
	free(priv);
}

static view_class vclass = {
	.view_id = VIEW_ACTION_MENU,
	.create = _create,
	.show = _show,
	.hide = _hide,
	.destroy = _destroy,
};

view_class *view_action_menu_get_vclass(void)
{
	return &vclass;
}
