/*

 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <app_debug.h>
#include <viewmgr.h>
#include <inputmgr.h>

#include "defs.h"
#include "view.h"
#include "datamgr.h"
#include "utils.h"

#define KEY_ICON "icon"
#define KEY_FOCUS_ICON "focusicon"

#define CTXPOPUP_X 294
#define CTXPOPUP_Y 310
#define MAX_BUF 128

enum input_handler_type {
	INPUT_HANDLER_TYPE_PHOTO,
	INPUT_HANDLER_TYPE_DELETE,
	INPUT_HANDLER_TYPE_EO,
	INPUT_HANDLER_TYPE_BTN_DONE,
	INPUT_HANDLER_TYPE_BTN_CANCEL,
	INPUT_HANDLER_TYPE_POPUP_BTN_DISCARD,
	INPUT_HANDLER_TYPE_POPUP_BTN_CANCEL,
	INPUT_HANDLER_TYPE_DELETE_POPUP_BTN_DELETE,
	INPUT_HANDLER_TYPE_DELETE_POPUP_BTN_CANCEL,
	INPUT_HANDLER_TYPE_ENTRY
};

enum user_item_type {
	USER_ITEM_NONE = 0x00,
	USER_ITEM_ADD = 0x01,
	USER_ITEM_ADMIN = 0x02,
	USER_ITEM_NORMAL = 0x08
};

struct _icon_info {
	const char *file;
	const char *focus_file;
	const char *message;
	const char *focus_message;
};

struct _priv {
	Evas_Object *win;
	Evas_Object *base;
	Evas_Object *ly;
	Evas_Object *photo;
	Evas_Object *name;
	Evas_Object *icon;
	Evas_Object *focus_icon;
	Evas_Object *done;
	Evas_Object *cancel;
	Evas_Object *ctxpopup;
	Evas_Object *del;
	Evas_Object *popup;

	char *selected_photo;

	struct datamgr *dm;
	struct datamgr_item *di;

	enum user_item_type user_type;
};

static void _focused(int id, void *data, Evas_Object *obj,
		Elm_Object_Item *item)
{
	struct _priv *priv;

	switch (id) {
	case INPUT_HANDLER_TYPE_PHOTO:
	case INPUT_HANDLER_TYPE_DELETE:
	case INPUT_HANDLER_TYPE_EO:
		elm_object_signal_emit(obj, SIG_FOCUS, SRC_PROG);
		break;
	case INPUT_HANDLER_TYPE_ENTRY:
		priv = data;
		elm_object_signal_emit(obj, SIG_SELECT, SRC_ELM);
		elm_object_signal_emit(priv->ly, SIG_FOCUS, SRC_PROG);
		break;
	default:
		 return;
	}
}

static void _unfocused(int id, void *data, Evas_Object *obj,
		Elm_Object_Item *item)
{
	struct _priv *priv;

	switch (id) {
	case INPUT_HANDLER_TYPE_PHOTO:
	case INPUT_HANDLER_TYPE_DELETE:
	case INPUT_HANDLER_TYPE_EO:
		elm_object_signal_emit(obj, SIG_UNFOCUS, SRC_PROG);
		break;
	case INPUT_HANDLER_TYPE_ENTRY:
		priv = data;
		elm_object_signal_emit(obj, SIG_UNSELECT, SRC_ELM);
		elm_object_signal_emit(priv->ly, SIG_UNFOCUS, SRC_PROG);
		break;
	default:
		return;
	}
}

static void _key_down(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Down *ev);

static void _mouse_up(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Up *ev);

static void _mouse_move(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Move *ev)
{
	if (!elm_object_focus_get(obj))
		elm_object_focus_set(obj, EINA_TRUE);
}

static input_handler handler = {
	.focused = _focused,
	.unfocused = _unfocused,
	.key_down = _key_down,
	.mouse_up = _mouse_up,
	.mouse_move = _mouse_move
};

static void _get_profile(struct _priv *priv, const char **name,
		const char **photo)
{
	*name = elm_entry_entry_get(priv->name);
	elm_image_file_get(priv->icon, photo, NULL);
	*photo = utils_get_icon_from_photo(*photo);
}

static void _delete_popup(struct _priv *priv, Evas_Object *obj)
{
	inputmgr_remove_callback(obj, &handler);
	evas_object_del(priv->popup);
	priv->popup = NULL;
}

static void _add_discard_popup(struct _priv *priv)
{
	Evas_Object *popup, *discard_btn, *cancel_btn;

	popup = utils_add_popup(priv->base, MESSAGE_DISCARD,
			MESSAGE_DISCARD_CONTENTS);
	if (!popup) {
		_ERR("failed to add popup");
		return;
	}
	priv->popup = popup;

	discard_btn = utils_add_button(popup, MESSAGE_DISCARD,
			PART_POPUP_BUTTON_1);
	if (!discard_btn) {
		_ERR("failed to add delete btn");
		evas_object_del(popup);
		return;
	}
	inputmgr_add_callback(discard_btn, INPUT_HANDLER_TYPE_POPUP_BTN_DISCARD,
			&handler, priv);

	cancel_btn = utils_add_button(popup, MESSAGE_CANCEL,
			PART_POPUP_BUTTON_2);
	if (!cancel_btn) {
		_ERR("failed to add cancel btn");
		evas_object_del(popup);
		return;
	}
	inputmgr_add_callback(cancel_btn, INPUT_HANDLER_TYPE_POPUP_BTN_CANCEL,
			&handler, priv);
	elm_object_focus_set(cancel_btn, EINA_TRUE);
}

static bool _check_changed_input(struct _priv *priv)
{
	const char *photo, *name;

	_get_profile(priv, &name, &photo);

	if (priv->di) {
		if (!strcmp(photo, priv->di->icon))
			return false;
	} else {
		if (!strcmp(name, "") && !strcmp(photo, IMAGE_USER_DEFAULT))
			return false;
	}

	return true;
}

static bool _add_btns(struct _priv *priv)
{
	Evas_Object *done, *cancel;

	done = utils_add_button(priv->base, MESSAGE_DONE,
			PART_USER_EDIT_BTN_DONE);
	if (!done) {
		_ERR("failed to add done btn");
		return false;
	}
	inputmgr_add_callback(done, INPUT_HANDLER_TYPE_BTN_DONE, &handler,
			priv);

	cancel = utils_add_button(priv->base, MESSAGE_CANCEL,
			PART_USER_EDIT_BTN_CANCEL);
	if (!cancel) {
		_ERR("failed to add cancel btn");
		evas_object_del(done);
		return false;
	}
	inputmgr_add_callback(cancel, INPUT_HANDLER_TYPE_BTN_CANCEL, &handler,
			priv);

	priv->cancel = cancel;
	priv->done = done;

	return true;
}

static void _load_user_icon(struct _priv *priv, const char *file,
		const char *focus_file)
{
	Evas_Object *icon, *focus_icon;

	icon = utils_add_icon(priv->photo, file, PART_USER_EDIT_PHOTO);
	if (!icon)
		_ERR("failed to add icon");

	focus_icon = utils_add_icon(priv->photo, focus_file,
			PART_USER_EDIT_PHOTO_FOCUS);
	if (!focus_icon)
		_ERR("failed to add focus icon");

	priv->icon = icon;
	priv->focus_icon = focus_icon;
}

static Evas_Object *_pack_icon(Evas_Object *table, struct icon_info *_icon_info,
		int i)
{
	Evas_Object *eo, *ic, *focus_ic;

	eo = utils_add_layout(table, GRP_USER_EDIT_ICON_LIST_ITEM, true, NULL);
	if (!eo) {
		_ERR("failed to add layout");
		return NULL;
	}
	evas_object_show(eo);
	elm_table_pack(table, eo, i % 4, i / 4, 1, 1);

	ic = utils_add_icon(eo, _icon_info->icon_file,
			PART_USER_EDIT_ICON_LIST_ITEM);
	if (!ic) {
		_ERR("failed to add icon");
		evas_object_del(eo);
		return NULL;
	}

	focus_ic = utils_add_icon(eo, _icon_info->focus_icon_file,
			PART_USER_EDIT_ICON_LIST_ITEM_FOCUS);
	if (!focus_ic) {
		_ERR("failed to add focus icon");
		evas_object_del(eo);
		return NULL;
	}
	evas_object_data_set(eo, KEY_ICON, _icon_info->photo_file);

	return eo;
}

static void _dismissed(void *data, Evas_Object *obj, void *ei)
{
	struct _priv *priv = data;

	elm_object_focus_allow_set(priv->photo, EINA_TRUE);
}

static void _add_icon_list(struct _priv *priv)
{
	Evas_Coord x, y, w, h;
	Evas_Object *ctxpopup, *ly, *table, *eo;
	int i;
	struct icon_info *_icon_info;

	ctxpopup = elm_ctxpopup_add(priv->base);
	if (!ctxpopup) {
		_ERR("failed to add ctxpopup");
		return;
	}
	elm_ctxpopup_horizontal_set(ctxpopup, EINA_TRUE);
	elm_ctxpopup_auto_hide_disabled_set(ctxpopup, EINA_TRUE);
	evas_object_smart_callback_add(ctxpopup, "dismissed", _dismissed, priv);

	ly = utils_add_layout(ctxpopup, GRP_USER_EDIT_ICON_LIST, false, NULL);
	if (!ly) {
		_ERR("failed to layout");
		evas_object_del(ctxpopup);
		return;
	}
	elm_object_content_set(ctxpopup, ly);
	evas_object_show(ly);
	evas_object_show(ctxpopup);

	evas_object_geometry_get(priv->photo, &x, &y, &w, &h);
	evas_object_move(ctxpopup,
			(x + w + CTXPOPUP_X) * elm_config_scale_get(),
			(y + CTXPOPUP_Y) * elm_config_scale_get());

	table = utils_add_table(ly, true, PART_USER_EDIT_ICON_LIST);
	if (!table) {
		_ERR("failed to add table");
		evas_object_del(ctxpopup);
		return;
	}
	elm_table_padding_set(table, elm_config_scale_get() * 1,
			elm_config_scale_get() * 1);
	evas_object_size_hint_weight_set(table, EVAS_HINT_EXPAND,
			EVAS_HINT_EXPAND);

	_icon_info = utils_get_icon_info();

	for (i = 0; i < MAX_ITEM_COUNT; i++) {
		eo = _pack_icon(table, &_icon_info[i], i);
		if (!eo)
			continue;

		inputmgr_add_callback(eo, INPUT_HANDLER_TYPE_EO, &handler,
				priv);
		if (i == 0)
			elm_object_focus_set(eo, EINA_TRUE);
	}

	elm_object_focus_allow_set(priv->photo, EINA_FALSE);
	priv->ctxpopup = ctxpopup;
}

static Evas_Object *_add_layout(struct _priv *priv)
{
	Evas_Object *ly, *photo, *name, *lock;

	ly = utils_add_layout(priv->base, GRP_USER_EDIT_CONTENTS, false,
			PART_USER_EDIT_CONTENTS);
	if (!ly) {
		_ERR("failed to add layout");
		return NULL;
	}

	photo = utils_add_layout(ly, GRP_USER_EDIT_PHOTO, true,
			PART_USER_EDIT_CONTENTS_PHOTO);
	if (!photo)
		goto err;

	name = utils_add_entry(ly, "", false, PART_USER_EDIT_CONTENTS_NAME);
	if (!name)
		goto err;

	lock = utils_add_layout(ly, GRP_USER_EDIT_SWITCH, false,
			PART_USER_EDIT_CONTENTS_LOCK);
	if (!lock)
		goto err;

	utils_add_icon(lock, IMAGE_SWITCH_OFF, PART_USER_EDIT_SWITCH);

	inputmgr_add_callback(photo, INPUT_HANDLER_TYPE_PHOTO, &handler, priv);
	inputmgr_add_callback(name, INPUT_HANDLER_TYPE_ENTRY, &handler, priv);

	priv->photo = photo;
	priv->name = name;

	return ly;
err:
	_ERR("failed to add layout");
	evas_object_del(ly);
	return NULL;
}

static bool _add_user_edit(struct _priv *priv)
{
	Evas_Object *ly;

	ly = _add_layout(priv);
	if (!ly) {
		_ERR("failed to add edit layout");
		return false;
	}

	if (!_add_btns(priv)) {
		_ERR("failed to add buttons");
		evas_object_del(ly);
		return false;
	}

	priv->ly = ly;

	return true;
}

static Evas_Object *_create(Evas_Object *win, void *data)
{
	struct _priv *priv;
	Evas_Object *base;

	if (!win || !data) {
		_ERR("Invalid argument");
		return NULL;
	}

	priv = calloc(1, sizeof(*priv));
	if (!priv) {
		_ERR("failed to calloc priv");
		return NULL;
	}

	base = utils_add_layout(win, GRP_USER_EDIT, false, NULL);
	if (!base) {
		_ERR("failed to create base");
		free(priv);
		return NULL;
	}
	evas_object_size_hint_weight_set(base, EVAS_HINT_EXPAND,
			EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, base);

	priv->win = win;
	priv->base = base;
	priv->dm = data;

	if (!_add_user_edit(priv)) {
		_ERR("failed to add user edit layout");
		evas_object_del(base);
		free(priv);
		return NULL;
	}

	viewmgr_set_view_data(VIEW_USER_EDIT, priv);
	viewmgr_add_view(view_photo_get_vclass(), NULL);
	viewmgr_add_view(view_action_menu_get_vclass(), NULL);

	return base;
}

static void _add_delete_popup(struct _priv *priv)
{
	Evas_Object *popup, *delete_btn, *cancel_btn;
	char buf[MAX_BUF];

	snprintf(buf, sizeof(buf), "%s %s?<br> %s", MESSAGE_DELETE,
			priv->di->title, MESSAGE_DELETE_CONTENTS);

	popup = utils_add_popup(priv->base, MESSAGE_DELETE_USER, buf);
	if (!popup) {
		_ERR("failed to add popup");
		return;
	}
	priv->popup = popup;

	delete_btn = utils_add_button(popup, MESSAGE_DELETE,
			PART_POPUP_BUTTON_1);
	if (!delete_btn) {
		_ERR("failed to add delete btn");
		evas_object_del(popup);
		return;
	}
	inputmgr_add_callback(delete_btn,
			INPUT_HANDLER_TYPE_DELETE_POPUP_BTN_DELETE,
			&handler, priv);

	cancel_btn = utils_add_button(popup, MESSAGE_CANCEL,
			PART_POPUP_BUTTON_2);
	if (!cancel_btn) {
		_ERR("failed to add cancel btn");
		evas_object_del(popup);
		return;
	}
	inputmgr_add_callback(cancel_btn,
			INPUT_HANDLER_TYPE_DELETE_POPUP_BTN_CANCEL,
			&handler, priv);

	elm_object_focus_allow_set(priv->del, EINA_FALSE);
	elm_object_focus_set(cancel_btn, EINA_TRUE);
}

static void _select_eo(struct _priv *priv, Evas_Object *obj)
{
	const char *file, *focus_file;

	evas_object_del(priv->ctxpopup);
	priv->ctxpopup = NULL;
	elm_object_focus_allow_set(priv->photo, EINA_TRUE);

	file = evas_object_data_get(obj, KEY_ICON);
	focus_file = utils_get_focus_photo_from_photo(file);

	if (!strcmp(file, ICON_ADD) || !strcmp(focus_file, ICON_ADD)) {
		viewmgr_push_view(VIEW_PHOTO);
		elm_object_focus_allow_set(obj, EINA_FALSE);
	} else
		_load_user_icon(priv, file, focus_file);
}

static void _select_btn_done(struct _priv *priv)
{
	const char *name, *photo;

	_get_profile(priv, &name, &photo);
	if (priv->di) {
		priv->di->action = ITEM_SELECT_ACTION_UPDATE;
		priv->di->title = strdup(name ? name : "");
		priv->di->icon = strdup(photo ? photo :
				IMAGE_USER_DEFAULT);
		datamgr_select_item(priv->dm, priv->di);
	} else {
		if (!name || !strcmp(name, "")) {
			elm_object_focus_set(priv->name, EINA_TRUE);
			return;
		}
		datamgr_add_item(priv->dm, name, photo, "");
	}

	priv->di = NULL;
	viewmgr_pop_view();
}

static void _select(struct _priv *priv, int id, Evas_Object *obj)
{
	switch (id) {
	case INPUT_HANDLER_TYPE_PHOTO:
		_add_icon_list(priv);
		break;
	case INPUT_HANDLER_TYPE_DELETE:
		_add_delete_popup(priv);
		break;
	case INPUT_HANDLER_TYPE_EO:
		_select_eo(priv, obj);
		break;
	case INPUT_HANDLER_TYPE_ENTRY:
		elm_object_focus_set(priv->done, EINA_TRUE);
		break;
	case INPUT_HANDLER_TYPE_BTN_DONE:
		_select_btn_done(priv);
		break;
	case INPUT_HANDLER_TYPE_BTN_CANCEL:
		if (_check_changed_input(priv)) {
			_add_discard_popup(priv);
			return;
		}
		viewmgr_pop_view();
		break;
	case INPUT_HANDLER_TYPE_DELETE_POPUP_BTN_DELETE:
		_delete_popup(priv, obj);
		elm_object_focus_allow_set(priv->del, EINA_TRUE);
		priv->di->action = ITEM_SELECT_ACTION_DELETE;
		datamgr_select_item(priv->dm, priv->di);
		viewmgr_pop_view();
		break;
	case INPUT_HANDLER_TYPE_POPUP_BTN_DISCARD:
		_delete_popup(priv, obj);
		viewmgr_pop_view();
		break;
	case INPUT_HANDLER_TYPE_DELETE_POPUP_BTN_CANCEL:
		elm_object_focus_allow_set(priv->del, EINA_TRUE);
	case INPUT_HANDLER_TYPE_POPUP_BTN_CANCEL:
		_delete_popup(priv, obj);
		break;
	default:
		return;
	}
}

static void _key_down(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Down *ev)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;
	if (!strcmp(ev->keyname, KEY_BACK) ||
			!strcmp(ev->keyname, KEY_BACK_REMOTE)) {
		switch (id) {
		case INPUT_HANDLER_TYPE_PHOTO:
		case INPUT_HANDLER_TYPE_BTN_DONE:
		case INPUT_HANDLER_TYPE_BTN_CANCEL:
		case INPUT_HANDLER_TYPE_DELETE:
		case INPUT_HANDLER_TYPE_ENTRY:
			if (_check_changed_input(priv)) {
				_add_discard_popup(priv);
				return;
			}
			viewmgr_pop_view();
			break;
		case INPUT_HANDLER_TYPE_EO:
			evas_object_del(priv->ctxpopup);
			priv->ctxpopup = NULL;
			elm_object_focus_allow_set(priv->photo, EINA_TRUE);
			break;
		case INPUT_HANDLER_TYPE_DELETE_POPUP_BTN_DELETE:
			_delete_popup(priv, obj);
			elm_object_focus_allow_set(priv->del, EINA_TRUE);
			break;
		case INPUT_HANDLER_TYPE_POPUP_BTN_DISCARD:
			_delete_popup(priv, obj);
			break;
		case INPUT_HANDLER_TYPE_DELETE_POPUP_BTN_CANCEL:
			elm_object_focus_allow_set(priv->del, EINA_TRUE);
		case INPUT_HANDLER_TYPE_POPUP_BTN_CANCEL:
			_delete_popup(priv, obj);
			break;
		default:
			return;
		}
	} else if (!strcmp(ev->keyname, KEY_ENTER) ||
			!strcmp(ev->keyname, KEY_ENTER_REMOTE)) {
		_select(priv, id, obj);
	} else if (!strcmp(ev->keyname, KEY_RIGHT)) {
		if (id == INPUT_HANDLER_TYPE_PHOTO && priv->del)
			elm_object_focus_set(priv->del, EINA_TRUE);
	} else if (!strcmp(ev->keyname, KEY_MENU) ||
			!strcmp(ev->keyname, KEY_MENU_REMOTE)) {
		viewmgr_show_view(VIEW_ACTION_MENU);
	}
}

static void _mouse_up(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Up *ev)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;
	_select(priv, id, obj);
}

static void _load_del_btn(struct _priv *priv)
{
	Evas_Object *btn;

	btn = utils_add_layout(priv->ly, GRP_USER_EDIT_DELETE, true,
			PART_USER_EDIT_CONTENTS_DELETE);
	if (!btn) {
		_ERR("failed to add delete button");
		return;
	}
	inputmgr_add_callback(btn, INPUT_HANDLER_TYPE_DELETE, &handler, priv);

	priv->del = btn;
}

static void _change_entry(Evas_Object *name, Evas_Object *ly, Eina_Bool disable,
		const char *text, const char *signal)
{
	elm_entry_entry_set(name, text);
	elm_object_focus_allow_set(name, disable);
	elm_entry_editable_set(name, disable);
	elm_object_signal_emit(ly, signal, SRC_PROG);
}

static void _show(void *data)
{
	struct _priv *priv;
	const char *photo, *focus_photo;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	if (priv->di) {
		elm_object_part_text_set(priv->base, PART_USER_EDIT_TITLE,
				MESSAGE_EDIT_USER);
		_change_entry(priv->name, priv->ly, EINA_FALSE, priv->di->title,
				SIG_DISABLE);
		if (priv->selected_photo) {
			_load_user_icon(priv, priv->selected_photo,
					priv->selected_photo);
		 } else {
			photo = utils_get_photo_from_icon(priv->di->icon);
			focus_photo = utils_get_focus_photo_from_photo(photo);
			_load_user_icon(priv, photo, focus_photo);
		}
		if (priv->user_type == USER_ITEM_ADMIN &&
				priv->di->type != USER_ITEM_ADMIN)
			_load_del_btn(priv);
	} else {
		elm_object_part_text_set(priv->base, PART_USER_EDIT_TITLE,
				MESSAGE_ADD_USER);
		_change_entry(priv->name, priv->ly, EINA_TRUE, "", SIG_ENABLE);
		if (priv->selected_photo)
			_load_user_icon(priv, priv->selected_photo,
					priv->selected_photo);
		else
			_load_user_icon(priv, IMAGE_USER_CURRENT_DEFAULT,
					IMAGE_USER_CURRENT_DEFAULT_FOCUS);
	}

	evas_object_show(priv->base);
	elm_object_focus_set(priv->cancel, EINA_TRUE);
}

static void _hide(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;
	if (priv->del) {
		inputmgr_remove_callback(priv->del, &handler);
		evas_object_del(priv->del);
		priv->del = NULL;
	}

	priv->selected_photo = NULL;
	evas_object_hide(priv->base);
}

static void _destroy(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	inputmgr_remove_callback(priv->photo, &handler);
	inputmgr_remove_callback(priv->done, &handler);
	inputmgr_remove_callback(priv->cancel, &handler);
	inputmgr_remove_callback(priv->name, &handler);
	viewmgr_remove_view(VIEW_PHOTO);
	evas_object_del(priv->base);
	free(priv);
}

static void _update(void *view_data, int update_type, void *data)
{
	struct _priv *priv;

	if (!view_data) {
		_ERR("Invalid argument");
		return;
	}

	priv = view_data;

	switch (update_type) {
	case UPDATE_TYPE_EDIT:
		priv->di = data;
		break;
	case UPDATE_TYPE_PHOTO:
		priv->selected_photo = data;
		break;
	case UPDATE_TYPE_USER:
		priv->user_type = (enum user_item_type) data;
		break;
	default:
		_ERR("Invalid type");
		return;
	}
}

static view_class vclass = {
	.view_id = VIEW_USER_EDIT,
	.create = _create,
	.show = _show,
	.hide = _hide,
	.destroy = _destroy,
	.update = _update
};

view_class *view_user_edit_get_vclass(void)
{
	return &vclass;
}
